/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <chrono>
#include <condition_variable>
#include <mutex>
#include <thread>

#include <string>
#include <kylin-ai/ai-engine/nlpengine.h>
#include <kylin-ai/ai-engine/ondevicenlpengine.h>

class LLM;

namespace ai_engine::nlp {

enum class TaskId {
    NORM_CHAT = -1,
    DOC_LABLE_EXTRACT,
    INTENT_RECOGNITION
};

class OndeviceNlpEngine : public AbstractOnDeviceNlpEngine {
public:
    OndeviceNlpEngine();
    ~OndeviceNlpEngine();
    std::string engineName() const override { return "ondevice"; }
    void setChatResultCallback(ChatResultCallback callback) override;
    void setContextSize(int size) override;
    void clearContext() override;
    // {
    //      //id:-1,norm chat;
    //      //id:0,lora_sq_doc_label_v1.0.0.gguf;
    //      //id:1,lora_sq_intent_v1.0.0.gguf
    //      //id:n,UNDO(norm chat);
    //     "task_id": 0, 
    //     "messages": [{
    //         "role": "system",
    //         "content": "你是一个AIPC的意图识别助理，你的任务是按照KYLIN_INTENT_DETECT的要求，
    //             结合上下文对话记录和用户输入进行意图识别和槽位信息抽取。"
    //     }, {
    //         "role": "user",
    //         "content": "请帮我调低电脑的亮度"
    //     }],
    //     "verbose": true
    // }
    virtual bool chat(const std::string &message,
                      ai_engine::EngineError &error) override;
    void stopChat() override;
    void onChatResult(const std::string &result, bool over);
    std::string getLocalModelList() const;
    virtual std::string modelInfo() const override;
    virtual bool setLlmModelName(const std::string &modelName, EngineError &error) override;

    virtual bool initChatModule(ai_engine::EngineError &error) override;
    virtual bool destroyChatModule(ai_engine::EngineError &error) override;

    virtual std::string currentModelName() const override;
    // virtual bool isCloud() const { return false; };
    virtual int maxConcurrentChatTasks() const { return 0; };
    uint taskId() const { return taskId_; }

private:
    void releaseTimerCallback();

private:
    uint taskId_ = 0;
    // int slotId_ = -1;
    uint sessionId_ = 0;
    bool inited_ = false;
    bool chatStopped_ = false;
    std::mutex releaseTimerMutex_;
    std::condition_variable releaseTimerCondition_;
    LLM *llm_;
    // NlpServer *nlpServer_ = nullptr;
    ChatResultCallback chatResultCallback_ = nullptr;
};

} // namespace ai_engine::nlp
